var escholar = (function () {

    return {

        searchBase : "https://www.escholar.manchester.ac.uk/api/",

        getDefaultQueryOptions : function () {
            return { searchBase: escholar.searchBase, json: true, rows: 10, start: 0, facets: [] };
        },

        search : function (query, resultsCallback, facetCallback) {
            var queryOptions = this.getDefaultQueryOptions();
            if( typeof query == "string") {
                queryOptions.q = query;
            } else {
                queryOptions = jQuery.extend(queryOptions, query);
            }

            $.each(queryOptions, function(key, value) {
                if (value == "" || value == undefined) {
                    delete queryOptions[key];
                }
            });

            $.getJSON(
                queryOptions.searchBase + 'search/?callback=?',//?q=' + queryOptions.q + "&start=" + queryOptions.start + facetQuery,
                queryOptions,
                function (data) {
                    var results = new escholar.SearchResults(data);
                    resultsCallback(results);
                });
            if(facetCallback != undefined) {
                $.getJSON(
                    // facet queries are slower so fetch them separately
                    queryOptions.searchBase + 'facet/?callback=?',//?q=' + queryOptions.q + facetQuery,
                    queryOptions,
                    function (data) {
                        var facets = new escholar.FacetSearchResult(data);
                        facetCallback(facets);
                    });
            }
        },

        getDocument : function(pid, documentCallback) {
            $.getJSON(
                this.getDefaultQueryOptions().searchBase + 'search/?callback=?q=PID%3A' + pid.replace(":", "\\:"),
                function (data) {
                    var results = new escholar.SearchResults(data);
                    documentCallback(results.getDocuments()[0]);
                });
        }
    };

})();

(function ( $ ) {
$.fn.eScholarSearch = function(query) {
    var resultsDiv = this;
    escholar.search(query, function(results) {

        $.each(results.getDocuments(), function(index, doc) {
            var object = $(document.createElement('div'));
            object.addClass("escholar.object");

            var link = $("<a>" + doc.title + "</a>");
            link.attr("href", doc.escholarPermLink);
            object.append(link);
                        
            var contributors = $(document.createElement('div'));
            contributors.addClass("escholar.contributors");
            contributors.text(doc.getContributors());
            object.append(contributors);

            var citation = $(document.createElement('div'));
            citation.addClass("escholar.citation");
            citation.text(doc.getCitationText());
            object.append(citation);

            var linkContainer = $(document.createElement("div"));
            linkContainer.addClass("escholar.permLink");
            linkContainer.append("eScholarID: ");
            var link = $(document.createElement("a"));
            link.attr("href", doc.escholarPermLink);
            link.text(doc.escholarId);
            linkContainer.append(link);
            object.append(linkContainer);

            if(doc.doiId != undefined) {
                linkContainer = $(document.createElement("div"));
                linkContainer.addClass("escholar.permLink");
                linkContainer.append("DOI: ");
                link = $(document.createElement("a"));
                link.addClass("escholar.doiPermLink");
                link.attr("href", doc.doiPermLink);
                link.text(doc.doiId);
                linkContainer.append(link);
                object.append(linkContainer);
            }
            resultsDiv.append(object);
        });
        return this;
    });
}}( jQuery ));
// Object representing a Document
// wrapper around JSON returned from solr
escholar.Document = function(doc) {
    this.title = doc["m.title"];
    this.publisher = doc["m.publisher"];
    this.year = doc["m.year"];

    this.resourceType = doc["m.typeofresource"]; // eg. text
    this.contentType = doc["r.isofcontenttype.source"].substr(0, doc["r.isofcontenttype.source"].indexOf('|')); // eg. Working paper
    this.escholarRef = doc["PID"];
    this.escholarId = doc["PID"].substr(doc["PID"].indexOf(':') + 1, doc["PID"].length -1);
    this.doiId = doc["m.identifier.doi"];

    this.doc = doc; // keep the whole doc for access to all fields
    this.getField = function(fieldName) {
        return doc[fieldName];
    };

    this.fileAttached = doc["f.isfileattached"] == "Yes";

    this.getFileLinks = function() {
        var files = new Array();
        $.each(escholar.Document.fileTypes, function(index, fileType) {
            if(fileType in doc) {
                var fileTypeElements = doc[fileType].split("||");
                var url = escholar.getDefaultQueryOptions.searchBase + "datastream?publicationPid=" + doc["PID"] + "&datastreamId=" + fileTypeElements[1];
                files.push({ "label" : fileTypeElements[0], "url": url, "mimeType": fileTypeElements[2] });
            }
        });
        return files;
    };

    //BA:[01/07/14] - add missing authors
    this.authors = this.getContributors();

    this.escholarPermLink = "https://www.escholar.manchester.ac.uk/" + this.escholarRef;
    this.doiPermLink = "http://dx.doi.org/" + doc["m.identifier.doi"];
};
escholar.Document.prototype.citationTemplates =  {
    "uk-ac-man-con:1" : function(doc) {
        //Journal article
        //m.host.title m.year;m.host.volume(m.host.issue):m.host.page.start-m.host.page.end
        var text =  doc["m.host.title"] + ". " + doc["m.year"] + ";" + doc["m.host.volume"];
        if(doc["m.host.issue"] != undefined ) { text += "(" + doc["m.host.issue"] + ")"; }
        if(doc["m.host.page.start"] != undefined) { text += ":" + doc["m.host.page.start"] + "-" + doc["m.host.page.end"] + "."; }
        else if(doc["m.host.page.list"] != undefined) { text += ": " + doc["m.host.page.list"] + "."; }
        return text;
    },
    "uk-ac-man-con:2" : function(doc) {
        //Conference contribution
        //In: m.note.editors, editor(s). m.host.title: m.host.host.title; m.host.host.dateother.start-m.host.host.dateother.end; m.host.host.placeterm. m.placeterm: m.publisher; m.year. p. m.host.page.start-m.host.page.end.
        var text = "In: ";
        if(doc["m.host.name.edt"] != undefined) { text += escholar.Document.printArray(doc["m.host.name.edt"], "; ") + ". "; }
        if(doc["m.note.editors"] != undefined) { text += doc["m.note.editors"] + ". "; }
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + ": "; }
        if(doc["m.host.host.title"] != undefined) { text += doc["m.host.host.title"] + "; "; }
        if(doc["m.host.host.dateother.start"] != undefined) {
            text += escholar.Document.getFormattedDate(doc["m.host.host.dateother.start"]) + "-"
                + escholar.Document.getFormattedDate(doc["m.host.host.dateother.end"])
                + "; ";
        }
        if(doc["m.host.host.placeterm"] != undefined) { text += doc["m.host.host.placeterm"] + ". "; }
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        if(doc["m.host.page.start"] != undefined) { text += " p. " + doc["m.host.page.start"] + "-" + doc["m.host.page.end"] + "."; }
        else if(doc["m.host.page.list"] != undefined) { text += " p. " + doc["m.host.page.list"] + "."; }
        return text;
    },
    "uk-ac-man-con:3" : function(doc) {
        //Book contribution
        //In: m.note.editors, editor(s). m.host.title: m.host.subtitle. m.host.edition ed. m.placeterm: m.publisher; m.year.
        var text = "In: ";
        if(doc["m.host.name.edt"] != undefined) { text += escholar.Document.printArray(doc["m.host.name.edt"], "; ") + ", editor(s). "; }
        if(doc["m.note.editors"] != undefined) { text += doc["m.note.editors"] + ", editor(s). "; }
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + ": "; }
        if(doc["m.host.subtitle"] != undefined) { text += doc["m.host.subtitle"] + ". "; }
        if(doc["m.host.edition"] != undefined) { text += doc["m.host.edition"] + " ed. "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        if(doc["m.host.page.start"] != undefined) { text += " p. " + doc["m.host.page.start"] + "-" + doc["m.host.page.end"] + "."; }
        else if(doc["m.host.page.list"] != undefined) { text += " p. " + doc["m.host.page.list"] + "."; }
        return text;
    },
    "uk-ac-man-con:4" : function(doc) {
        //Book
        //m.host.edition ed. m.placeterm: m.publisher; m.year.
        var text = "";
        if(doc["m.host.edition"] != undefined) { text += doc["m.host.edition"] + " ed. "; }
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;
    },
    "uk-ac-man-con:5" : function(doc) {
        //Report
        // m.placeterm: m.publisher; m.year. Report No.: m.identifier.reportnumber.
        var text = "";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        if(doc["m.identifier.reportnumber"] != undefined) { text += " Report No.: " + doc["m.identifier.reportnumber"] + "."; }
        return text;
    },
    "uk-ac-man-con:6" : function(doc) {
        //Conference proceeding
        // m.host.title; m.host.dateother.start-m.host.dateother.end; m.host.placeterm. m.placeterm: m.publisher; m.year.
        var text = "";
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + "; "; }
        if(doc["m.host.dateother.start"] != undefined) {
            text += escholar.Document.getFormattedDate(doc["m.host.dateother.start"]) + "-"
                + escholar.Document.getFormattedDate(doc["m.host.dateother.end"])
                + "; ";
        }
        if(doc["m.host.placeterm"] != undefined) { text += doc["m.host.placeterm"] + ". "; }
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;},
    "uk-ac-man-con:7" : function(doc) {
        //Composition
        // m.otherformat.title: m.otherformat.placeterm; m.otherformat.dateother. m.placeterm: m.publisher; m.year.
        var text = "";
        if(doc["m.otherformat.title"] != undefined) { text += doc["m.otherformat.title"] + ": "; }
        if(doc["m.otherformat.placeterm"] != undefined) { text += doc["m.otherformat.placeterm"] + "; (premiere). "; }
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;},
    "uk-ac-man-con:8" : function() {
        // no documents with this identifier
        return "";
    },
    "uk-ac-man-con:9" : function(doc) {
        //Dissertation
        // [Dissertation]. m.placeterm: m.publisher; m.year.
        var text = "[Dissertation].";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + ";"; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;},
    "uk-ac-man-con:10" : function(doc) {
        //Exhibition
        // m.placeterm. m.dateother.start-m.dateother.end.
        var text = "";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + "."; }
        if(doc["m.host.host.dateother.start"] != undefined) {
            text += " " + escholar.Document.getFormattedDate(doc["m.host.host.dateother.start"]) + "-"
                + escholar.Document.getFormattedDate(doc["m.host.host.dateother.end"])
                + ".";
        }
        return text;},
    "uk-ac-man-con:11" : function(doc) {
        //Exhibition contribution
        //In: m.host.title: m.host.placeterm. m.host.dateother.start-m.host.dateother.end.
        var text = "In: ";
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + ": "; }
        if(doc["m.host.placeterm"] != undefined) { text += doc["m.host.placeterm"] + "."; }
        if(doc["m.host.host.dateother.start"] != undefined && doc["m.host.host.dateother.end"] != undefined) {
            text += " " + escholar.Document.getFormattedDate(doc["m.host.host.dateother.start"]) + "-"
                + escholar.Document.getFormattedDate(doc["m.host.host.dateother.end"])
                + ". ";
        }
        return text;
    },
    "uk-ac-man-con:12" : function(doc) {
        // no documents with this identifier
        return text;
    },
    "uk-ac-man-con:13" : function(doc) {
        //Patent
        // m.note.patentassignees, m.publisher. m.identifier.patentnumber. m.dateissued. p. m.host.page.start-m.host.page.end
        var text = "";
        if(doc["m.note.patentassignees"] != undefined) { text += doc["m.note.patentassignees"] + ", "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + ". "; }
        if(doc["m.identifier.patentnumber"] != undefined) { text += doc["m.identifier.patentnumber"] + ". "; }
        if(doc["m.dateissued"] != undefined) { text += doc["m.dateissued"] + "."; }
        if(doc["m.host.page.start"] != undefined) { text += " p. " + doc["m.host.page.start"] + "-" + doc["m.host.page.end"]; }
        else if(doc["m.host.page.list"] != undefined) { text += " p. " + doc["m.host.page.list"]; }
        return text;
    },
    "uk-ac-man-con:14" : function(doc) {
        //Technical report
        // m.placeterm: m.publisher; m.year. Report No.: m.identifier.reportnumber.
        var text = "";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        if(doc["m.identifier.reportnumber"] != undefined) { text += " Report No.: " + doc["m.identifier.reportnumber"] + "."; }
        return text;
    },
    "uk-ac-man-con:15" : function(doc) {
        //Thesis
        // [Thesis]. m.placeterm: m.publisher; m.year.
        var text = "[Thesis]. ";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;
    },
    "uk-ac-man-con:16" : function(doc) {
        // Working paper
        // m.host.title. m.placeterm: m.publisher; m.year. Working paper no. m.identifier.workingpapernumber.
        var text = "";
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + ". "; }
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + "; "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + ". "; }
        if(doc["m.identifier.workingpapernumber"] != undefined) { text += "Working Paper No. " + doc["m.identifier.workingpapernumber"] + "."; }
        return text;
    },
    "uk-ac-man-con:17" : function(doc) {
        //Journal contribution
        //
        var text = "";
        return text;
    },
    "uk-ac-man-con:18" : function(doc) {
        //Newspaper/magazine contribution
        // m.host.title. m.dateissued;m.host.volume(m.host.issue).m.host.section:m.host.page.start-m.host.page.end
        var text = "";
        if(doc["m.host.title"] != undefined) { text += doc["m.host.title"] + ". "; }
        if(doc["m.dateissued"] != undefined) { text += doc["m.dateissued"] + "; "; }
        if(doc["m.host.volume"] != undefined) { text += doc["m.host.volume"]; }
        if(doc["m.host.issue"] != undefined) { text += "(" + doc["m.host.issue"] + ")"; }
        if(doc["m.host.section"] != undefined) { text += "." + doc["m.host.section"] + ": "; }
        if(doc["m.host.page.start"] != undefined) { text += doc["m.host.page.start"] + "-" + doc["m.host.page.end"]; }
        else if(doc["m.host.page.list"] != undefined) { text += doc["m.host.page.list"]; }
        return text;
    },
    "uk-ac-man-con:19" : function(doc) {
        //Lite-cite submission
        // m.genre.version. m.year.
        var text = "";
        if(doc["m.genre.version"] != undefined) { text += doc["m.genre.version"] + ". "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + ". "; }
        return text;
    },
    "uk-ac-man-con:20" : function(doc) {
        //Administered thesis
        //[Thesis]. m.placeterm: m.publisher; m.year.
        var text = "[Thesis]. ";
        if(doc["m.placeterm"] != undefined) { text += doc["m.placeterm"] + ": "; }
        if(doc["m.publisher"] != undefined) { text += doc["m.publisher"] + "; "; }
        if(doc["m.year"] != undefined) { text += doc["m.year"] + "."; }
        return text;
    }
};

escholar.Document.prototype.getCitationText = function() {
    var type = this.doc["r.isofcontenttype.pid"];
    return this.citationTemplates[type](this.doc);
};

escholar.Document.prototype.getContrib = function (primary, secondary, tertiary) {
    if (this.doc[primary] != undefined) {
    	//BA: [12/11/14] - Fix problem of jumbled up names - start
    	var strPrimary = this.doc[primary].toString();
    	//BA: Global replace of || seems to have undesirable effect. Using this clumsy method instead
    	while(strPrimary.indexOf("||") > -1){
    		strPrimary = strPrimary.replace("||", "; ");
    	}    	
    	strPrimary = strPrimary.replace(/ ,/g, " ");
    	strPrimary = strPrimary.replace(/ ;/g, "");
    	if(strPrimary.substring(strPrimary.length - 2, strPrimary.length - 1) == ";"){
    		strPrimary = strPrimary.substring(0, strPrimary.length - 2);
    		strPrimary += ".";
    	}
        //return this.doc[primary];
        return strPrimary;
    	//BA: [12/11/14] - Fix problem of jumbled up names - end
    } else if (this.doc[secondary] != undefined) {
    	return [this.doc[secondary]];
    } else if (this.doc[tertiary] != undefined) {
    	return [this.doc[tertiary]];
    }
};

escholar.Document.prototype.getContributors = function() {
    if ("uk-ac-man-con:6" == this.doc["r.isofcontenttype.pid"]) {
        return this.getContrib("m.host.name.edt", "m.note.editors");
    } else if ("uk-ac-man-con:10" == this.doc["r.isofcontenttype.pid"]) {
        return this.getContrib("m.name.cur", "m.note.curators", "m.note.authors");
    } else if ("uk-ac-man-con:11" == this.doc["r.isofcontenttype.pid"]) {
        return this.getContrib("m.name.cre", "m.note.creators");
    } else if ("uk-ac-man-con:13" == this.doc["r.isofcontenttype.pid"]) {
        return this.getContrib("m.name.pth", "m.note.patentholders");
    } else {
    	//BA: [12/11/14] - Fix problem of jumbled up names - start
        //return this.getContrib("m.name.aut", "m.note.authors");
        return this.getContrib("m.name.aut.source", "m.note.authors");
    	//BA: [12/11/14] - Fix problem of jumbled up names - end
    }
};

escholar.Document.prototype.getContributorsText = function() {
    return escholar.Document.printArray(this.getContributors());
};

escholar.Document.printArray = function(array, separator) {
    var text = "";
    if (array == undefined) {
        return text;
    }
    if (separator == undefined) {
        separator = "; ";
    }
    $.each(array, function(index, item) {
        text += item;
        if (index + 1 < array.length) {
            text += separator;
        }
    });
    return text;
};

escholar.Document.getFormattedDate = function(dateString) {
    var date = new Date(dateString);
    var months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
    return (date.getDate() < 10 ? "0" : "") + date.getDate() + " " + months[date.getMonth()] + " " + date.getFullYear();
};


escholar.Document.fileTypes = [
    "f.pre-peer-review.source",
    "f.post-peer-review-non-publishers-document.source",
    "f.post-peer-review-publishers.source",
    "f.post-peer-review-publishers-document.source",
    "f.full-text.source",
    "f.supplementary-1.source",
    "f.supplementary-2.source",
    "f.supplementary-3.source",
    "f.supplementary-4.source",
    "f.supplementary-5.source",
    "f.supplementary-5.source",
    "f.supplementary-6.source",
    "f.supplementary-7.source",
    "f.supplementary-8.source",
    "f.supplementary-9.source",
    "f.supplementary-10.source"
];
escholar.FacetSearchResult = function(data) {
    this.data = data;
    var facetMap = {
        "m.ispeerreviewed":"Peer-reviewed scholarly work",
        "f.isfileattached":"Records with full-text and/or other file(s) deposited in eScholar",
        "r.isofcontenttype.source.facet":"Content type",
        "m.name.aut.facet":"Author",
        "m.year":"Publish year",
        "m.topic.facet":"Keyword",
        "r.isbelongsto.source.facet":"University researcher",
        "r.isbelongstoorg.source.facet":"Academic department"
    };

    var FacetGroup = function(id, label, facets) {
        this.id = id;
        this.label = label;
        this.facets = facets;
    };

    var Facet = function(id, name, count) {
        this.id = id;
        this.name = name;
        this.count = count;
    };

    var facets = function() {
        var facetGroups = new Array();

        $.each(data.facet_counts.facet_fields, function(facetGroupName){
            var facetGroupLabel = facetMap[facetGroupName];
            var facets = new Array();

            $.each(data.facet_counts.facet_fields[facetGroupName], function(facetTitle){
                var facetDetails = facetTitle.split("||");
                var facetId = (facetDetails.length > 1) ? facetDetails[1] : facetDetails[0];
                var facet = new Facet(facetId, facetDetails[0], data.facet_counts.facet_fields[facetGroupName][facetTitle]);
                facets.push(facet);
            });

            facetGroups.push(new FacetGroup(facetGroupName, facetGroupLabel, facets));
        });
        return facetGroups;
    };

    return {
        getFacets: function() {
            return facets();
        }
    }
};
escholar.SearchResults = function(result) {
    this.result = result;

    var docs = function() {
        var docs = new Array();

        $.each(result.response.docs, function(index, doc) {
            docs.push(new escholar.Document(doc));
        });
        return docs;
    };

    return {
        getQuery: function() {
            return result.responseHeader.params.q;
        },
        getResultCount: function() {
            return parseInt(result.response.numFound);
        },
        getQueryTime: function() {
            return parseInt(result.responseHeader.QTime);
        },
        getRowCount: function() {
            return parseInt(result.responseHeader.params.rows);
        },
        getStart: function() {
            return parseInt(result.responseHeader.params.start);
        },
        getDocuments: function() {
            return docs();
        },
        getPrevResultsQuery: function() {
            if(this.getStart() > 1) {
                return { q: this.getQuery(), start: (this.getStart() - this.getRowCount()), rows: this.getRowCount() };
            } else {
                return { q: this.getQuery(), start: this.getStart(), rows: this.getRowCount() };
            }
        },
        getNextResultsQuery: function() {
            if(this.getStart() > 1) {
                return { q: this.getQuery(), start: (this.getStart() - this.getRowCount()), rows: this.getRowCount() };
            } else {
                return { q: this.getQuery(), start: this.getStart(), rows: this.getRowCount() };
            }
        }
    }
};
